#!/bin/sh
enum() # An easy way to add numbers to lists
{
  num=1
  while read line; do
    echo "${num}. ${line}"
    num="`expr "$num" + 1`"
  done
}
die() # Error handling
{
  echo "Error: $1"
  exit 1
}
# Find disks, and their sizes to make it easier to find the right one
disks=''
for disk in /dev/hd? /dev/sd?; do # is hd* even used anymore?
  if echo "$disk" | grep -q '?'; then continue; fi
  size="`blockdev --getsize64 "$disk"`"
  suffix='bytes'
  if [ "$size" -gt 1024 ]; then
    size="`expr "$size" / 1024`"
    suffix='kb'
    if [ "$size" -gt 1024 ]; then
      size="`expr "$size" / 1024`"
      suffix='mb'
      if [ "$size" -gt 1024 ]; then
        size="`expr "$size" / 1024`"
        suffix='gb'
        if [ "$size" -gt 1024 ]; then
          size="`expr "$size" / 1024`"
          suffix='tb'
        fi
      fi
    fi
  fi
  disks="${disks}
${disk}    ${size}${suffix}"
done
disks="`echo "$disks" | grep . | enum`" # Filter out unwanted empty line
if [ "$disks" = '' ]; then die "Your computer doesn't seem to have a harddrive :/"; fi
# Ask the user which disk to use (optionally partition)
disk=''
while [ "$disk" = '' ]; do
  echo 'Please choose a disk:'
  echo "$disks"
  read -p '> ' num
  disk="`echo "$disks" | sed -n -e "${num}p" 2> /dev/null`"
done
dev="`echo "$disk" | sed -e 's/^[^ ]* //;s/ .*//'`"
echo "Picked device '${dev}'"
# Ask how to partition it
options='1. Make one large partition for everything, overwriting any old data.
2. Manually partition with cfdisk.
3. No partition, make the entire disk one large filesystem. (I'"'"'m not sure grub likes this)'
option=''
while [ "$option" = '' ]; do
  echo "Please choose how you wish to partition ${dev}:"
  echo "$options"
  read -p '> ' num
  option="`echo "$options" | sed -n -e "${num}p" 2> /dev/null`"
done
case "$option" in
  *'one large partition'*)
    (
    echo o # New partition table
    echo n # New partition
    echo p # Primary
    echo 1 # Parititon 1
    echo 2048 # First sector
    echo   # Last sector, default is end of disk
    echo w # Write changes to disk and exit
    ) | fdisk "$dev" > /dev/null;;
  *'Manually partition'*) cfdisk "$dev";;
  # *'No partition'*) ;; # Do nothing, or maybe we should do something to erase unwanted partitions here
esac
# Ask which partition to use as root
options="`ls "${dev}"?* 2> /dev/null | enum`" # Get list of partitions
if [ "$options" = '' ]; then
  option="$dev"
else
  option=''
  while [ "$option" = '' ]; do
    echo 'Please choose which partition to use as root:'
    echo "$options"
    read -p '> ' num
    option="`echo "$options" | sed -n -e "${num}p" 2> /dev/null`"
  done
fi
part="`echo "$option" | sed -e 's/.* //'`"
echo "Picked partition '${part}'"
# Choose filesystem
options="`ls /usr/sbin/mkfs.* /sbin/mkfs.* 2> /dev/null | sed -e 's/.*\.//;/ext4dev/d' | enum`"
option=''
while [ "$option" = '' ]; do
  echo "Please choose a filesystem to format ${part} as:"
  echo "$options"
  read -p '> ' num
  option="`echo "$options" | sed -n -e "${num}p" 2> /dev/null`"
done
fs="`echo "$option" | sed -e 's/.* //'`"
echo "Running mkfs.${fs} ${part}"
"mkfs.${fs}" "$part"
# Basically just follow /etc/motd
# 1. mount root
mount "$part" /mnt || die "Failed to mount ${part}"
# 2. create buildtmp
mkdir -p /mnt/buildtmp
# 4. cd to buildtmp
cd /mnt/buildtmp
# Cleanup, just in case
rm -rf root
/root/buildaos.sh -c
# 3. create root symlink
ln -s /mnt root
# 5. create src and symlinks
mkdir -p src
ln -s /usr/src/* src
# 6. run buildaos.sh
rm -f buildaos.sh
wget http://aos.ion.nu/files/buildaos.sh || cp /root/buildaos.sh .
chmod +x buildaos.sh
cores="`sed -n -e '/^cpu cores/{s/.* //;p;q;}' /proc/cpuinfo`"
# TODO: more options? maybe offer the --nowifi flag or -p for different architectures, or find out if the CPU supports 64-bit and make an assumption?
# Note: We can find if long-mode (64) is supported with this command: cat /proc/cpuinfo | grep -q '^flags.*:.* lm\b'
./buildaos.sh -m "$cores"
# 7. install grub
grub-install --boot-directory=/mnt/boot "$dev" || die "Failed to install grub on ${dev}"
# TODO: copy boot_bg.png and set it up as a grub background? No, we need to make an alternative one that doesn't say (LiveCD) in that case
# cp /boot/boot_bg.png /mnt/boot
# sed -i -e '/set timeout/abackground_image /boot/boot_bg.png' /mnt/boot/grub/grub.cfg
# We also need a font with UTF8 border characters, e.g. ucs-fonts. can be converted to grub's format with grub-mkfont -o ucs.pf2 -s 14 7x14.bdf
# 8. unmount root
cd /
umount /mnt
# 9. done
echo "Done! type 'reboot' to reboot and start using your new AOS install (don't forget to remove the liveCD/USB)"
