#!/bin/sh
# Prefix for 32-bit buildtools, e.g. "i586-pc-linux-gnu-", can also be an absolute path in case the tools are not in PATH
prefix32=""

timestamp="`date +%s`" # Used to uniquely identify this AOS live build to itself

# Should be run as root, I guess
if [ ! -e findlivecd.c ]; then echo "findlivecd.c is missing"; exit 1; fi
if ! which "${prefix32}gcc" > /dev/null 2> /dev/null; then echo "Can't find 32-bit toolchain"; exit 1; fi
if ! echo 'int main(){}' | "${prefix32}gcc" -m32 -x c - -o /dev/null 2> /dev/null; then echo "Toolchain can't build 32-bit executables, get a 32-bit toolchain"; exit 1; fi
if ! which mksquashfs > /dev/null 2> /dev/null; then echo "mksquashfs is missing (pkg: squashfs)"; exit 1; fi
if [ ! -e rootfs.tar.gz ]; then echo "Obtain a root filesystem and store as rootfs.tar.gz"; exit 1; fi
if [ "`id -u`" != "0" ]; then echo "Run as root (can't use proot because we're building emacs)"; exit 1; fi
umount root/dev 2> /dev/null # Just in case
umount root/proc 2> /dev/null
rm -rf root syslinux-6.02 aos_live.iso # Clean slate

if [ "$1" = "fast" ]; then
  if [ ! -e rootfs_fast.tar.gz ]; then echo "You must do a slow build first to create the _fast cache."; exit 1; fi
  tar --no-same-owner -xzf rootfs_fast.tar.gz
else
tar --no-same-owner -xzf rootfs.tar.gz
cat > root/root/makelive.sh << END
#!/bin/sh
echo 'nameserver 208.67.222.222' > /etc/resolv.conf
cd
wget http://aos.ion.nu/files/wiki2shell.sh
chmod +x wiki2shell.sh
# AOS build dependencies
./wiki2shell.sh patch | sh -e
./wiki2shell.sh m4 | sh -e
./wiki2shell.sh bc | sh -e
./wiki2shell.sh flex | sh -e
./wiki2shell.sh bison | sh -e
./wiki2shell.sh perl | sh -e
./wiki2shell.sh gettext | sh -e
# Optional/outside dependencies
./wiki2shell.sh tmux | sh -e
./wiki2shell.sh grub | sh -e
# Filesystem tools
./wiki2shell.sh e2fsprogs | sh -e
./wiki2shell.sh dosfstools | sh -e
./wiki2shell.sh hfsprogs | sh -e
./wiki2shell.sh ntfs-3g | sh -e
# TODO: add more filesystem tools (to allow formatting the root with different filesystems)
./wiki2shell.sh unionfs-fuse | sh -e
# Editors
./wiki2shell.sh vim | sh -e
./wiki2shell.sh emacs | sh -e
./wiki2shell.sh nano | sh -e
# Probing tools
./wiki2shell.sh links | sh -e
./wiki2shell.sh procps | sh -e
./wiki2shell.sh man-db | sh -e
./wiki2shell.sh man-pages | sh -e
./wiki2shell.sh pciutils | sh -e # Good for probing your system
./wiki2shell.sh usbutils | sh -e
# Miscellaneous
./wiki2shell.sh openssh | sh -e
./wiki2shell.sh testdisk | sh -e
rm -rf /usr/src/* /usr/parent-src # We won't need these on the finished system
rm -rf /lib/modules/*/build /lib/modules/*/source
# rm -f /etc/installedpackages # Leave it in
wget http://aos.ion.nu/files/buildaos.sh
chmod +x buildaos.sh
cd /usr/src
sed -n -e '/\(^wgetwrapper\|; *wgetwrapper\) /{s/ *#.*//;s/wgetwrapper/wget --no-check-certificate -c/g;s/.*; *//;s/).*//;p;}' /root/buildaos.sh | sh
END
chmod +x root/root/makelive.sh
mount /dev root/dev -o bind
mount /proc root/proc -o bind
i386 chroot root /root/makelive.sh
umount root/dev
umount root/proc
rm -f rootfs_fast.tar.gz
tar -czf rootfs_fast.tar.gz root
fi # fast
rm -f root/root/makelive.sh
rm -f root/root/wiki2shell.sh
rm -f root/root/.bash_history
rm -f root/etc/resolv.conf
rm -rf root/boot/grub
mkdir -p root/mnt
sed -i -e 's/    \(GNU+Linux\)/\1 LiveCD/' root/etc/motd
(
echo
echo 'To install AOS, follow these instructions:'
echo '0. Partition and format the target disk (you can use cfdisk and the mkfs.* commands for this)'
echo '1. Mount the partition on which you wish to install AOS on at /mnt (assumed to be clean formatted)'
echo '2. Create the directory /mnt/buildtmp (because we probably need more space than a RAM-disk can get us)'
echo '3. Create a symlink from /mnt/buildtmp/root to /mnt (using ln -s /mnt /mnt/buildtmp/root)'
echo '4. Change directory to /mnt/buildtmp'
echo '5. Create the directory src and symlink sources from /usr/src (ln -s /usr/src/* src)'
echo '6. Run /root/buildaos.sh with the desired flags (see --help), or download the latest version from http://aos.ion.nu/files/buildaos.sh and use that'
echo '7. Install grub, using the command grub-install --boot-directory=/mnt/boot /dev/your_harddrive (note: not partition)'
echo '8. Change directory back to /root (or anywhere not within /mnt) and remove /mnt/buildtmp and unmount /mnt'
echo '9. Reboot'
echo
echo 'If you need to see these instructions again, run cat /etc/motd'
echo 'Alternatively you can run the command "installguide" for a simple guided setup (experimental)'
) >> root/etc/motd
echo 'AOS-LiveCD' > root/etc/hostname
# export PS1='[\u@\h] \w\$ '
# A shell prompt with return status indicator
echo "export PS1='"'[\[[32m\]\u@\h\[[0m\]] \[[34m\]\w\[[0m`[ "$?" -ne "0" ] && echo "[31m"`\]\$\[[0m\] '"'" > root/root/.bashrc
ln -s .bashrc root/root/.profile

# Strip executables
(
"${prefix32}strip" root/lib/*.so
"${prefix32}strip" root/bin/*
"${prefix32}strip" root/sbin/*
"${prefix32}strip" root/usr/lib/*.so
"${prefix32}strip" root/usr/bin/*
"${prefix32}strip" root/usr/sbin/*
) 2> /dev/null

# Split up init so that it can be continued within a writable chroot
sed -n -e '1p; /^hostname/,$p' root/sbin/init > root/sbin/init2
chmod +x root/sbin/init2
sed -i -e '/^hostname/,$d' root/sbin/init
cat >> root/sbin/init << 'END'
  mount -t proc none /proc
  mount -t tmpfs none /tmp
  if ! grep -q ' rw ' /proc/cmdline && ! grep -q ' rw$' /proc/cmdline; then
    umount /proc 2> /dev/null # Let init2 mount it
    cp -r /etc/. /tmp/etc_rw
    mount /tmp/etc_rw /etc -o bind
    cp -r /root/. /tmp/root_rw
    mount /tmp/root_rw /root -o bind
  else
    mkdir /tmp/root_rw
    mkdir /tmp/root_ro
    mkdir /tmp/root
    mount / /tmp/root_ro -o bind
    unionfs -o dirs=/tmp/root_rw=rw:/tmp/root_ro=ro,cow /tmp/root
#    exec chroot /tmp/root /sbin/init2 # I don't remember how/why, but I guess changing it to the below solution solved the shutdown issue
    sed -i -e 's|/sbin/init shell|chroot /tmp/root /bin/bash|' /tmp/root/sbin/init2
    sed -i -e 's|^mount[^/]* /|&tmp/root/|' /tmp/root/sbin/init2
    sed -i -e 's|^mkdir[^/]* /|&tmp/root/|' /tmp/root/sbin/init2
    exec /tmp/root/sbin/init2
  fi
  exec /sbin/init2
END

sed -i -e '/remount,rw/d' root/sbin/init2 # We can't remount the CD as writable, and removing the line gets rid of an annoying message

# Write a simple guided install
cp installguide.sh root/usr/bin/installguide
chmod +x root/usr/bin/installguide

# Compress the filesystem
mksquashfs root root.squash -no-exports -no-xattrs -all-root -comp xz
for x in root/*; do
  if [ "$x" = "root/boot" ]; then continue; fi # Spare a few files
  rm -rf "$x"
done
mv root.squash root

# Build and install syslinux
wget -c https://www.kernel.org/pub/linux/utils/boot/syslinux/syslinux-6.02.tar.xz
tar -xJf syslinux-6.02.tar.xz
cd syslinux-6.02
make firmware=bios -j4
cd ..
mkdir -p root/boot/isolinux
cp syslinux-6.02/bios/core/isolinux.bin root/boot/isolinux
cp syslinux-6.02/bios/com32/chain/chain.c32 root/boot/isolinux
cp syslinux-6.02/bios/com32/gfxboot/gfxboot.c32 root/boot/isolinux
cp syslinux-6.02/bios/com32/menu/menu.c32 root/boot/isolinux
cp syslinux-6.02/bios/memdisk/memdisk root/boot/isolinux
cp syslinux-6.02/bios/com32/elflink/ldlinux/ldlinux.c32 root/boot/isolinux
cp syslinux-6.02/bios/com32/libutil/libutil.c32 root/boot/isolinux

cp syslinux-6.02/bios/com32/menu/vesamenu.c32 root/boot/isolinux
cp syslinux-6.02/bios/com32/lib/libcom32.c32 root/boot/isolinux

cp boot_bg.png root/boot
kernel="`ls root/boot/vmlinuz-* | sed -e 's|.*/||'`"
cat > root/boot/isolinux/isolinux.cfg << END
TIMEOUT 100
UI vesamenu.c32
MENU BACKGROUND /boot/boot_bg.png
MENU COLOR sel 37;44 #ffffffff #403050ff standard

DEFAULT 1024x768

LABEL 1280x1024
MENU LABEL AOS LiveCD 1280x1024
LINUX /boot/${kernel}
APPEND vga=794 quiet
INITRD /boot/live-initrd-${timestamp}

LABEL 1024x768
MENU LABEL AOS LiveCD 1024x768
LINUX /boot/${kernel}
APPEND vga=792 quiet
INITRD /boot/live-initrd-${timestamp}

LABEL 800x600
MENU LABEL AOS LiveCD 800x600
LINUX /boot/${kernel}
APPEND vga=789 quiet
INITRD /boot/live-initrd-${timestamp}

LABEL 640x480
MENU LABEL AOS LiveCD 640x480
LINUX /boot/${kernel}
APPEND vga=785 quiet
INITRD /boot/live-initrd-${timestamp}

LABEL nofb
MENU LABEL AOS LiveCD no framebuffer
LINUX /boot/${kernel}
APPEND quiet
INITRD /boot/live-initrd-${timestamp}

LABEL rw
MENU LABEL AOS LiveCD experimental writable root
LINUX /boot/${kernel}
APPEND quiet rw
INITRD /boot/live-initrd-${timestamp}
END

# Build init for initrd
"${prefix32}gcc" -static -DTIMESTAMP="\"${timestamp}\"" findlivecd.c -o findlivecd
"${prefix32}strip" findlivecd

# Create initrd image
rm -rf tmp_initrd
mkdir tmp_initrd
mkdir tmp_initrd/dev
mkdir tmp_initrd/proc
mkdir tmp_initrd/.cdrom
mv findlivecd tmp_initrd/init
cd tmp_initrd
find . | cpio --create --format=newc > "../root/boot/live-initrd-${timestamp}"
cd ..
rm -rf tmp_initrd
cp findlivecd.c root/boot # Provide source

mkisofs -o aos_live.iso -b boot/isolinux/isolinux.bin -c boot/isolinux/boot.cat -no-emul-boot -boot-load-size 4 -boot-info-table -allow-lowercase -allow-multidot -iso-level 4 -l -f -relaxed-filenames -R -no-iso-translate root
syslinux-6.02/bios/utils/isohybrid aos_live.iso # Make it bootable from USB stick
